# -*- coding: utf-8 -*-
# SDK info
__version__ = "0.1.11"
__package_name__ = "fastflyer"
__title__ = "FastFlyer — 轻量级API开发框架"

__author__ = "Jagerzhang"
__author_email__ = "im@zhang.ge"

__license__ = "Apache 2.0"

__python_requires__ = ">=3, <4"
__classifiers__ = ["Programming Language :: Python :: 3"]
__url__ = "https://github.com/jagerzhang/fastflyer.git"

__copyright__ = "Copyright © 2024 Tencent. All Rights Reserved. 网络平台部 平台研发中心 版权所有"
__description__ = "FastFlyer 是基于 FastAPI 设计的轻量级 API 开发框架。在 FastAPI 优异特性的基础上集成了一系列开箱即用的组件，\
包括Redis、MySQL、Kafka等，让研发人员只需要专注于业务逻辑的实现，真正开箱即用！"



# -*- coding: utf-8 -*-
"""
聚合路由
"""
# -*- coding: utf-8 -*-
from fastflyer import APIRouter
from .userinfo import router as user_router
from .relationship import router as relate_router

router = APIRouter()
router.include_router(user_router, tags=["用户管理"], prefix="/users")
router.include_router(relate_router, tags=["用户管理"], prefix="/users")



# -*- coding: utf-8 -*-
from os import getenv, path
import uvicorn
from fastflyer import FlyerAPI

root_path = path.dirname(path.abspath(__file__))
app_path = f"{root_path}/app"
app = FlyerAPI(app_path=app_path)


def main():
    """
    通过shell执行uvicorn启动
    """
    pass


def main_cmd():
    """
    直接执行 main.py 启动
    """
    bind_host = getenv("flyer_host", "0.0.0.0")
    bind_port = getenv("flyer_port", 8080)
    workers = int(getenv("flyer_workers", 1))
    reload = int(getenv("flyer_reload", 0))
    log_level = getenv("flyer_console_log_level", "info")
    access_log = True if int(getenv("flyer_access_log", "1")) else False
    uvicorn.run(app="main:app",
                host=bind_host,
                port=int(bind_port),
                log_level=log_level,
                access_log=access_log,
                reload=reload,
                workers=workers)


if __name__ == "__main__":
    main_cmd()



"""示例任务
"""
from fastflyer import logger, background_scheduler, threadpool


# 定时任务示例：每5秒执行一次
@background_scheduler.scheduled_job('interval', seconds=5, single_job=True)
def hello_world():
    logger.info("hello world by background-scheduler every 5 senconds!")


# 后台线程执行示例
# @threadpool.submit()  # 支持装饰器添加方式
def hello_world_thread():
    # 直到线程池停止才结束循环
    while not threadpool.is_stopped():
        logger.warning("hello world by threadpool every 5 senconds!")
        threadpool.sleep(5)



# -*- coding: utf-8 -*-
"""
任务管理
"""
from fastapi import APIRouter
from fastflyer import status, background_scheduler, config
from fastflyer.schemas import DataResponse, BaseResponse
from fastflyer.middleware import MiddleWare

router = APIRouter(tags=["任务管理"], prefix=config.PREFIX, route_class=MiddleWare)


# pylint: disable=invalid-name
@router.get("/tasks", response_model=DataResponse, summary="任务查询接口")
def get_jobs():
    """查询所有任务
    """
    jobs = background_scheduler.get_jobs()
    job_list = []
    for job in jobs:
        job_info = {
            "id": job.id,
            "name": job.name,
            "nextRunTime": job.next_run_time,
            "trigger": job.trigger.__class__.__name__,
            "args": job.args,
            "kwargs": job.kwargs,
            "misfireGraceTime": job.misfire_grace_time,
            "maxInstances": job.max_instances
        }
        job_list.append(job_info)
    return {"data": job_list}


@router.get("/tasks/{taskId}", response_model=DataResponse, summary="查询指定任务")
def get_job(taskId: str):
    """
    查询指定任务
    """
    job = background_scheduler.get_job(taskId)
    if job:
        job_info = {
            "id": job.id,
            "name": job.name,
            "next_run_time": job.next_run_time,
            "trigger": job.trigger.__class__.__name__,
            "args": job.args,
            "kwargs": job.kwargs,
            "misfire_grace_time": job.misfire_grace_time,
            "max_instances": job.max_instances
        }
        return {"data": job_info}
    else:
        return {
            "code": status.HTTP_404_NOT_FOUND,
            "message": f"Job {taskId} not found."
        }


@router.post("/tasks/{taskId}", response_model=BaseResponse, summary="启动指定任务")
def start_job(taskId: str):
    """
    启动指定任务
    """
    job = background_scheduler.get_job(taskId)
    if job:
        if not job.next_run_time:  # 检查任务是否已停止
            background_scheduler.resume_job(taskId)
            return {"message": f"Job {taskId} has been started."}
        else:
            return {"message": f"Job {taskId} is already running."}
    else:
        return {
            "code": status.HTTP_404_NOT_FOUND,
            "message": f"Job {taskId} not found."
        }


@router.delete("/tasks/{taskId}",
               response_model=BaseResponse,
               summary="停止任务接口")
def stop_job(taskId: str):
    """
    停止指定任务
    """
    job = background_scheduler.get_job(taskId)
    if job:
        background_scheduler.remove_job(taskId)
        return {
            "code": status.HTTP_200_OK,
            "message": f"Job {taskId} has been stopped."
        }
    else:
        return {
            "code": status.HTTP_404_NOT_FOUND,
            "message": f"Job {taskId} not found."
        }


@router.delete("/tasks", response_model=DataResponse, summary="停止所有任务")
def stop_jobs():
    """
    停止所有任务
    """
    jobs = background_scheduler.get_jobs()
    stopped_jobs = []
    for job in jobs:
        # 框架本身任务不被停止
        if job.id == "polaris_heartbeat" or job.id.startswith("sync_config_"):
            continue
        background_scheduler.remove_job(job.id)
        stopped_jobs.append({"id": job.id, "name": job.name})

    return {"data": stopped_jobs}


# pylint: disable=invalid-name



from os import getenv
from fastflyer import config
from opentelemetry import trace
from opentelemetry.exporter.otlp.proto.grpc.trace_exporter import OTLPSpanExporter
from opentelemetry.sdk.trace import TracerProvider
from opentelemetry.sdk.trace import ReadableSpan
from opentelemetry.sdk.trace.export import BatchSpanProcessor
from opentelemetry.sdk.resources import Resource
from opentelemetry.instrumentation.logging import LoggingInstrumentor
from opentelemetry.instrumentation.httpx import HTTPXClientInstrumentor
from opentelemetry.instrumentation.requests import RequestsInstrumentor
from opentelemetry.instrumentation.kafka import KafkaInstrumentor
from opentelemetry.instrumentation.pymysql import PyMySQLInstrumentor
from opentelemetry.instrumentation.sqlalchemy import SQLAlchemyInstrumentor
from opentelemetry.instrumentation.redis import RedisInstrumentor
from opentelemetry.instrumentation.threading import ThreadingInstrumentor
from opentelemetry.trace import SpanKind
from fastkit.logging import get_logger

logger = get_logger("console")


class MySpanProcessor(BatchSpanProcessor):
    """自定义处理，以修复trace拆分多条上报的问题
    """

    def on_end(self, span: ReadableSpan) -> None:
        """丢弃多于的阶段trace（需配置开关：flyer_opentelemetry_reduce_request_trace=1）
        """
        is_reduce = int(getenv("flyer_opentelemetry_reduce_request_trace", 0))
        if is_reduce == 1 and span.kind == SpanKind.INTERNAL and (
                span.attributes.get('asgi.event.type')
                in ('http.request', 'http.response.start',
                    'http.response.body', 'http.disconnect')):
            return
        super().on_end(span=span)

    def paas_instrument(self):
        """paas组件上报opentelemetry
        """
        # http客户端
        if int(getenv("flyer_opentelemetry_http_enabled", 1)) == 1:
            HTTPXClientInstrumentor().instrument(
                request_hook=httpx_request_hook,
                async_request_hook=httpx_async_request_hook)
            RequestsInstrumentor().instrument(
                request_hook=httpx_request_hook,
                async_request_hook=httpx_async_request_hook)
        # kafka 客户端
        if int(getenv("flyer_opentelemetry_kafka_enabled", 1)) == 1:
            KafkaInstrumentor().instrument()
        # sqlalchemy 客户端
        if int(getenv("flyer_opentelemetry_sqlachemy_enabled", 1)) == 1:
            SQLAlchemyInstrumentor().instrument(enable_commenter=True)
        # pymysql 客户端（默认关闭，避免和SQLAlchemyInstrumentor重复）
        if int(getenv("flyer_opentelemetry_pymysql_enabled", 0)) == 1:
            PyMySQLInstrumentor().instrument()
        # Redis 客户端
        if int(getenv("flyer_opentelemetry_redis_enabled", 1)) == 1:
            RedisInstrumentor().instrument(request_hook=redis_request_hook)


def setup_opentelemetry():
    """初始化opentelemetry
    """
    if int(getenv("flyer_opentelemetry_enabled", 0)) != 1:
        return None

    tenant_id = getenv("flyer_opentelemetry_tenant_id",
                       getenv("TPS_TENANT_ID", ""))
    service_name = getenv("flyer_opentelemetry_service_name",
                          config.API_TITLE.replace(" ", ""))
    endpoint = getenv(
        "flyer_opentelemetry_endpoint",
        "http://otel-collect-proxy.zhiyan.tencent-cloud.net:4317")

    if not tenant_id:
        logger.warning("未设置租户ID，无法上报 Opentelemetry 监控数据！")
        return None

    logger.info(
        "已启用 Opentelemetry 监控数据上报：service_name=%s, tenant_id=%s, endpoint=%s",
        service_name, tenant_id, endpoint)

    resource = Resource(attributes={
        "service.name": service_name,
        "tps.tenant.id": tenant_id
    })
    tracer = TracerProvider(resource=resource)
    trace.set_tracer_provider(tracer)
    processor = MySpanProcessor(span_exporter=OTLPSpanExporter(
        endpoint=endpoint))
    tracer.add_span_processor(processor)
    LoggingInstrumentor().instrument()
    # 多线程兼容
    ThreadingInstrumentor().instrument()
    if tracer:
        processor.paas_instrument()

    return tracer


def httpx_request_hook(span, request):
    """修改httpx的spanname
    """
    method = request.method.decode() if isinstance(request.method,
                                                   bytes) else request.method
    url = str(request.url)
    span.update_name(f"{method} {url}")


async def httpx_async_request_hook(span, request):
    """修改httpx的spanname
    """
    method = request.method.decode() if isinstance(request.method,
                                                   bytes) else request.method
    url = str(request.url)
    span.update_name(f"{method} {url}")


def fastapi_server_request_hook(span, request):
    method = request.get("method")
    path = request.get("path")
    span.update_name(f"{method} {path}")


def fastapi_client_request_hook(span, request, message):
    method = request.get("method")
    path = request.get("path")
    span.update_name(f"{method} {path} http receive")


def fastapi_client_response_hook(span, request, message):
    method = request.get("method")
    path = request.get("path")
    if message.get("type") == "http.response.start":
        span.update_name(f"{method} {path} http response start")
    else:
        span.update_name(f"{method} {path} http response end")


def redis_request_hook(span, instance, args, kwargs):
    if len(args) > 1:
        operation = args[0]
        key = args[1]
        span.update_name(f"{operation} {key}")



# -*- coding: utf-8 -*-
"""
路由文件
"""
from fastflyer import status
from fastflyer.schemas import DataResponse
from fastflyer import APIRouter

router = APIRouter(tags=["项目管理"])


@router.get("/items/{id}", response_model=DataResponse, summary="项目查询接口")
async def get_item(id: int):
    """
    演示接口：项目信息查询
    ---
    - 附加说明2: 这个位置可以加入更多说明列表。
    """
    result = {"code": status.HTTP_200_OK, "data": {"itemId": id}}
    return result



# -*- coding: utf-8 -*-
"""
用户关系路由
"""
from fastflyer import status
from fastflyer.schemas import DataResponse
from fastflyer import APIRouter

router = APIRouter()


@router.get("/relationship/{username}",
            response_model=DataResponse,
            summary="关系查询接口")
async def relationship(username: str):
    """
    演示接口：查询用户关系
    ---
    - 附加说明1: 仅用于演示，请勿对号入座；
    - 附加说明2：这个位置可以加入更多说明列表。
    """
    data = {"father": f"{username}'s father", "mother": f"{username}'s mother"}
    result = {"code": status.HTTP_200_OK, "msg": "", "data": data}
    return result



"""管理工具
"""

import os
import sys
import glob
import shutil
import argparse
from datetime import datetime
import pkg_resources
from fastkit.logging import get_logger
from fastflyer import threadpool

logger = get_logger(logger_name="console")
COLOR_RED = "\033[91m"
COLOR_GREEN = "\033[92m"
COLOR_YELLOW = "\033[93m"
COLOR_DEFAULT = "\033[39m"
COLOR_RESET = "\033[0m"


def colored_cover(message, color=COLOR_YELLOW):
    return f"{color}{message}{COLOR_RESET}"


def main():
    try:
        parser = argparse.ArgumentParser(description="FastFlyer 框架开发辅助工具")
        subparsers = parser.add_subparsers(dest="command")

        # 修改为 create 命令
        create_parser = subparsers.add_parser("create", help="创建应用代码")
        create_parser.add_argument("type", choices=["openapi"], help="选择创建类型")
        create_parser.add_argument("--name", help="指定API名称")
        create_parser.add_argument("--prefix", help="指定API接口路径前缀，默认为 /flyer")
        create_parser.add_argument("-f", "--force", action="store_true", help="强制覆盖已存在文件")

        # 修改为 show 命令
        show_parser = subparsers.add_parser("show", help="显示相关信息")
        show_subparsers = show_parser.add_subparsers(dest="show_command")

        show_subparsers.add_parser("openapi", help="快速启动 OpenAPI 示例项目")

        args = parser.parse_args()

        if args.command == "create":
            if args.type in ["openapi"]:
                if os.path.exists(".init_lock") and not args.force:
                    logger.warn("项目已经被初始化过，请勿重复初始化")
                else:
                    template_dir = pkg_resources.resource_filename(__name__, f"template/{args.type}")
                    copy_files(template_dir, ".", args.force)

                    # 读取 --name 和 --prefix 参数
                    name = args.name if args.name else "Flyer Demo"
                    prefix = args.prefix if args.prefix else "/flyer"

                    # 替换 settings.py 文件中的 API_TITLE 和 PREFIX
                    settings_file = f"{template_dir}/settings.py"
                    with open(settings_file, "r") as f:
                        filedata = f.read()
                    filedata = filedata.replace('API_TITLE = "Flyer Demo"', f'API_TITLE = "{name}"')
                    filedata = filedata.replace(
                        'PREFIX = getenv("flyer_base_url", "/flyer")', f'PREFIX = getenv("flyer_base_url", "{prefix}")'
                    )
                    with open(settings_file, "w") as f:
                        f.write(filedata)

                    if os.path.exists(".init_lock"):
                        os.remove(".init_lock")
                    with open(".init_lock", "w") as lock_file:
                        lock_file.write(str(datetime.now()))
                    logger.info(
                        f"初始化完成，你可以执行 ls 命令查看目录内容或执行{colored_cover('./dev_ctrl.sh')}快速构建开发环境"
                    )

        elif args.command == "show":
            if args.show_command == "openapi":
                logger.info("尝试读取相关环境变量...")
                for key, value in os.environ.items():
                    if not key.startswith("flyer_"):
                        continue
                    logger.info(f"{key}={value}")

                from fastflyer.tools.template.openapi.main import main_cmd
                from fastflyer.utils import get_host_ip

                port = os.getenv("flyer_port", "8080")
                prefix = os.getenv("flyer_base_url", "/flyer")
                url = f"http://{get_host_ip()}:{port}{prefix}"
                logger.info("欢迎启动 FastFlyer OpenAPI 体验项目，现在可以通过浏览器访问以下项目页面：")
                logger.info(f"SwaggerUI 文档：{colored_cover(f'{url}/docs')}")
                logger.info(f"ReDoc 接口文档：{colored_cover(f'{url}/redoc')}")
                main_cmd()

            else:
                parser.print_help()

        else:
            # 如果没有指定命令，则输出帮助信息
            parser.print_help()

    finally:
        # 在所有情况下都调用 shutdown
        threadpool.shutdown(wait=False, show_log=False)


def copy_files(source_dir, dest_dir, force=False):
    if not os.path.exists(dest_dir):
        os.makedirs(dest_dir)

    copied_files = []  # 用于跟踪已经拷贝过的文件路径

    for root, dirs, files in os.walk(source_dir):
        for dir in dirs:
            if dir == "__pycache__":
                continue
            src_dir = os.path.join(root, dir)
            dest_subdir = os.path.join(dest_dir, os.path.relpath(src_dir, source_dir))

            is_copied = False
            for copied_dir in copied_files:
                if dest_subdir.startswith(copied_dir):
                    is_copied = True
                    break

            if is_copied:
                continue

            if not force and os.path.exists(dest_subdir):
                logger.warn(f"目录 {dest_subdir} 已存在. 请添加 --force 参数执行强制覆盖")
            else:
                if sys.version_info >= (3, 8):
                    shutil.copytree(src_dir, dest_subdir, dirs_exist_ok=True)
                else:
                    shutil.copytree(src_dir, dest_subdir)
                copied_files.append(dest_subdir)  # 添加已拷贝的目录路径到列表

        for file in files:
            if file.endswith((".pyc", ".log", ".lock")):
                continue  # 排除后缀为 .pyc 和 .log 的文件
            src_file = os.path.join(root, file)
            dest_file = os.path.join(dest_dir, os.path.relpath(src_file, source_dir))

            is_copied = False
            for copied_dir in copied_files:
                if dest_file.startswith(copied_dir):
                    is_copied = True
                    break

            if is_copied:
                continue

            if not force and os.path.exists(dest_file):
                logger.warn(f"文件 {dest_file} 已存在. 请添加 --force 参数执行强制覆盖")
            else:
                shutil.copy2(src_file, dest_file)

    # 删除当前目录下以 .log 结尾的文件
    for file in glob.glob("*.log"):
        os.remove(file)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
import fcntl
import os
import socket
import struct
import sys


def get_client_ip(request):
    """get the client IPAddress by request
    """
    try:
        real_ip = request.headers['X-Forwarded-For']
        if len(real_ip.split(',')) > 1:
            client_ip = real_ip.split(",")[0]

        else:
            client_ip = real_ip

    except Exception:  # pylint: disable=broad-except
        client_ip = request.client.host

    return client_ip


def get_host_ip():
    """get the host IPAddress by socket
    """
    host_ip = None
    try:
        socket_fd = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        socket_fd.connect(("www.qq.com", 80))
        host_ip = socket_fd.getsockname()[0]

    finally:
        socket_fd.close()

    if host_ip:
        return host_ip

    try:
        host_ip = get_ip_by_interface("eth1")

    except OSError:
        try:
            host_ip = get_ip_by_interface("eth0")

        except OSError:
            host_ip = None

    return host_ip


def get_ip_by_interface(ifname):
    """获取指定网卡的IP地址
    """
    ifname = ifname[:15]
    if sys.version_info.major == 3:
        ifname = bytes(ifname, "utf-8")

    socket_fd = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    ip_addr = socket.inet_ntoa(
        fcntl.ioctl(
            socket_fd.fileno(),
            0x8915,  # SIOCGIFADDR
            struct.pack("256s", ifname))[20:24])
    return ip_addr


def get_env_list(prefix=None, replace=True):
    """ 获取环境变量
        @ prefix： 指定目标变量的前缀
        @ replace：指定前缀后，键名是否去掉前缀
    """
    env_dict = os.environ

    if prefix:
        env = {}
        for key in env_dict:
            if prefix in key:
                if replace:
                    env[key.replace(prefix, "")] = env_dict[key]
                else:
                    env[key] = env_dict[key]

        return env

    else:
        return dict(env_dict)


def get_log_path(config=None):
    """
    自动获取日志目录
    """
    if hasattr(config, "LOG_DIR"):
        log_path = config.LOG_DIR
    else:
        log_path = f"{os.getcwd()}/logs"

    if not os.path.exists(log_path):
        os.makedirs(log_path)

    return log_path



from os import getenv, environ
from fastkit.logging import get_logger  # noqa

log_path = getenv("flyer_log_path", "/var/log")
logger = get_logger(logger_name="console",
                    file_log_level=getenv("flyer_file_log_level"),
                    console_log_level=getenv("flyer_console_log_level"),
                    log_path=log_path)

logger.info("已加载环境变量如下：")
for k, v in environ.items():
    if len(v.split("\n")) > int(getenv("flyer_max_wrap_line") or 10):
        v = v.replace("\n", "\\n")
    logger.info(f"{k}: {v}")



# -*- coding: utf-8 -*-
#
# 异常处理相关常量及工具函数等
from traceback import format_exc
from typing import Any
from httptools.parser.errors import HttpParserInvalidMethodError
from fastapi import FastAPI
from fastapi import status as fastapiStatus, HTTPException
from fastapi.responses import JSONResponse
from fastapi.exceptions import RequestValidationError
from pydantic import ValidationError
from fastflyer import config, status
from fastkit.logging import get_logger

# 定义返回码基数，最小值为0，默认为 0，和HTTP返回码保持一致
STATUS_CODE_BASE = max(getattr(config, "STATUS_CODE_BASE", 0), 0)

logger = get_logger(logger_name="console", log_path=config.LOG_PATH)


def init_exception(app: FastAPI):
    """初始化异常处理"""

    def get_detail(message: str) -> str:
        return message.strip().split("\n")[-1]

    # pylint: disable=unused-variable
    @app.exception_handler(HttpParserInvalidMethodError)
    async def handle_invalid_http_method_error(request, exc):
        return ErrorResponse(status_code=status.HTTP_400_BAD_REQUEST,
                             message="非法的请求方法",
                             detail=str(exc))

    @app.exception_handler(RequestValidationError)
    async def validation_exception_handler(request, exc: Exception):
        """请求参数异常"""
        return ErrorResponse(status.HTTP_400_BAD_REQUEST,
                             message="请求参数校验不通过",
                             detail=str(exc))

    @app.exception_handler(ValidationError)
    async def resp_validation_exception_handler(request, exc: Exception):
        """响应值参数校验异常"""
        return ErrorResponse(status.HTTP_403_FORBIDDEN,
                             message="响应参数校验不通过",
                             detail=str(exc))

    @app.exception_handler(BaseException)
    async def base_exception_handler(request, exc: BaseException):
        """捕获自定义异常"""
        # 把异常的详细信息打印到控制台，也可以在此实现将日志写入到对应的文件系统等
        logger.warning(format_exc())
        return ErrorResponse(exc.code, message=exc.message, detail=exc.detail)

    @app.exception_handler(HTTPException)
    async def http_exception_handler(request, exc: HTTPException):
        """捕获FastAPI异常"""
        logger.warning(format_exc())
        return ErrorResponse(exc.status_code,
                             message=str(exc.detail),
                             detail=exc.detail)

    @app.exception_handler(Exception)
    async def allexception_handler(request, exc: Exception):
        """捕获所有其他的异常
        """
        message = format_exc()
        logger.warning(message)
        return ErrorResponse(status.HTTP_500_INTERNAL_SERVER_ERROR,
                             message="内部异常",
                             detail=get_detail(message))

    @app.exception_handler(KeyError)
    async def keyerror_handler(request, exc: KeyError):
        """捕获所有其他的异常"""
        message = format_exc()
        logger.warning(message)
        return ErrorResponse(status.HTTP_500_INTERNAL_SERVER_ERROR,
                             message="内部异常",
                             detail=get_detail(message))

    @app.exception_handler(ValueError)
    async def valueerror_handler(request, exc: ValueError):
        """捕获所有其他的异常"""
        message = format_exc()
        logger.warning(message)
        return ErrorResponse(status.HTTP_500_INTERNAL_SERVER_ERROR,
                             message="内部异常",
                             detail=get_detail(message))

    # pylint: disable=unused-variable


# 状态码对应的异常信息(默认)
messages = {
    # 4XX
    status.HTTP_400_BAD_REQUEST: "请求参数校验不通过",
    status.HTTP_401_UNAUTHORIZED: "权限校验不通过",
    status.HTTP_403_FORBIDDEN: "响应参数校验不通过",
    status.HTTP_404_NOT_FOUND: "请求的资源不存在",
    # 5XX
    status.HTTP_500_INTERNAL_SERVER_ERROR: "服务器内部错误",
    status.HTTP_504_GATEWAY_TIMEOUT: "请求上游服务时超时",
}


class BaseException(HTTPException):
    """自定义异常基类
    程序内部抛出的异常应该给予该基类
    异常都使用这个类型或者其子类进行抛出，会被统一进行处理和响应。
    对于嵌套的异常处理，如果捕获到这个类型的，则直接raise即可，其他的异常则可以进行进一步的处理。
    """

    def __init__(self,
                 code: int,
                 message: str = None,
                 detail: Any = None) -> None:
        """
        :param code 必须是在在status中定义好的值
        :param message 异常信息，通常可以展示给前端用户看
        :param detail 详细异常信息，通常是用于开发排查问题
        """
        self.code = code
        self.message = message if message else messages[code]
        status_code = code if code < 600 else fastapiStatus.HTTP_500_INTERNAL_SERVER_ERROR
        super().__init__(status_code, detail)

    def __str__(self) -> str:
        return f"code={self.code} message={self.message}\n detail={self.detail}"


class InternalException(BaseException):
    """内部错误异常
    异常时通常使用该类型进行raise
    """
    pass


class ErrorResponse(JSONResponse):
    """接口异常响应类型
    通常只需要在中间件捕获异常的时候使用。
    异常时可以指定一个状态，这个状态码应该尽量重用http标准的状态码，
    对于超过范围的值，可以定义到600到999的范围，大于等于600的时候，
    在响应时会自动重置为500.
    响应给前端的异常信息结构(假设SYSTEM_CODE_BASE的值为1000)：
    Example1:
    {
        "code": 1401,
        "message": "这是自定义错误信息"
    }
    这时http响应的状态码应该时401
    Example2:
    {
        "code": 1602,
        "message": "这是自定义错误信息"
    }
    这时http响应的状态码应该时500（大于等于600时自动重置为500）

    其中：
    code值是完整的异常状态码，message是异常描述信息。
    """
    """
        :param code 响应状态码，正常取值0-999，若该值与1000的余数大于等于600，则http code会自动重置为500。若该值大于等于1000，则该值可能来自上游接口
        :param message 异常信息，通常是用于展示给用户。如果该值为空，则会默认为code值对应的异常信息
        :param detail 详细的异常信息，通常用于开发者排除定位问题使用
        """

    def __init__(self,
                 code: int,
                 message: str = None,
                 detail: Any = None,
                 content: dict = None,
                 status_code: int = 200) -> None:
        """通用JSON响应类

        Args:
            code (int): Body 响应码，200为正常，其他为异常
            message (str, optional): 异常信息. Defaults to None.
            detail (Any, optional): 详细的异常信息. Defaults to None.
            content (dict, optional): 正常请求下的JSON响应内容. Defaults to None.
            status_code (int, optional): HTTP响应码，默认一直为200. Defaults to 200.
        """
        base_content = {}
        content = content or {}
        base_content["code"] = code
        base_content["message"] = message if message else messages.get(
            code, "")
        base_content["detail"] = detail if detail else ""

        base_content.update(content)

        if code == fastapiStatus.HTTP_200_OK:
            super().__init__(status_code=status_code, content=base_content)
            return

        # http的状态码大于600会报错，超过600响应为内部错误
        status_code = status_code if status_code < 600 else status.HTTP_500_INTERNAL_SERVER_ERROR
        # 消息返回码叠加一下基数
        base_content["code"] = STATUS_CODE_BASE + code
        super().__init__(status_code=status_code, content=base_content)


if __name__ == "__main__":
    resp = ErrorResponse(status.HTTP_403_FORBIDDEN, message="接口请求参数错误")
    logger.info(resp.body)



# -*- coding: utf-8 -*-
"""
用户信息路由
"""
from fastflyer import status
from fastflyer.schemas import DataResponse
from fastflyer import APIRouter
from app.users.schemas.userinfo import UserCreateRequest

router = APIRouter()


@router.post("/info", response_model=DataResponse, summary="用户录入接口")
async def create_user(params: UserCreateRequest):
    """
    演示接口：批量录入用户信息
    ---
    - 附加说明2: 这个位置可以加入更多说明列表。
    """
    result = {"code": status.HTTP_200_OK, "message": "", "data": params.data}
    return result



# -*- coding: utf-8 -*-
from fastflyer.__info__ import *  # noqa

# 优先加载业务config，如果不存在则加载默认config
try:
    from settings import CustomConfig as config
except ImportError:
    from fastflyer.settings import BaseConfig as config

from fastkit.utils.environ import EnvObject
from fastkit.logging import logger  # noqa

# httpx 的opentelemetry 需要提前加
from fastflyer.trace import setup_opentelemetry

tracer = setup_opentelemetry()

from fastkit.httpx import status  # noqa
from fastkit.httpx import Client  # noqa
from fastkit.httpx import AsyncClient  # noqa
from fastflyer.scheduler import background_scheduler, asyncio_scheduler, threadpool  # noqa

from .exceptions import ErrorResponse  # noqa
from .main import FlyerAPI  # noqa
from .router import APIRouter  # noqa

# 环境变量对象
env = EnvObject()

# HTTP 同步请求客户端
client = requests = Client(logger=logger, **config.DEFAULT_REQUEST_RETRY_CONFIG)
# HTTP 异步请求客户端
aioclient = aiorequests = AsyncClient(logger=logger, **config.DEFAULT_REQUEST_RETRY_CONFIG)



from fastapi import APIRouter as FastAPIRouter
# from fastapi.routing import APIRoute
from .middleware import MiddleWare


class APIRouter(FastAPIRouter):

    def __init__(self, *args, **kwargs):
        super().__init__(route_class=MiddleWare, *args, **kwargs)



"""任务调度
"""
from os import getenv
from apscheduler.schedulers.background import BackgroundScheduler
from apscheduler.schedulers.asyncio import AsyncIOScheduler
from fastkit.executor import Scheduler, ThreadPoolExecutor
from fastkit.logging import get_logger
from fastflyer import config

logger = get_logger(logger_name="console", log_path=config.LOG_PATH)

redis_host = getenv("flyer_redis_host")
REDIS_CONFIG = None
if redis_host:
    redis_port = int(getenv("flyer_redis_port", "6379"))
    redis_pass = getenv("flyer_redis_pass", "")
    redis_db = int(getenv("flyer_redis_db", "10"))
    REDIS_CONFIG = {
        "host": redis_host,
        "port": redis_port,
        "passwd": redis_pass,
        "database": redis_db
    }

max_threads = int(getenv("flyer_threads", "5"))
# 线程型后台任务调度
background_scheduler: BackgroundScheduler = Scheduler(
    redis_config=REDIS_CONFIG,
    name="fastflyer",
    scheduler_type="background",
    logger=logger,
    auto_start=False,
    executor_type="threadpool",
    pool_size=max_threads)

# 协程型后台任务调度
asyncio_scheduler: AsyncIOScheduler = Scheduler(redis_config=REDIS_CONFIG,
                                                name="fastflyer",
                                                scheduler_type="asyncio",
                                                logger=logger,
                                                auto_start=False,
                                                executor_type="threadpool",
                                                pool_size=max_threads)

# 线程池后台任务
threadpool = ThreadPoolExecutor(redis_config=REDIS_CONFIG)



from os import getenv
from fastflyer.settings import BaseConfig


class CustomConfig(BaseConfig):
    """
    自定义配置

    Args:
        BaseConfig (_type_): 框架默认配置
    """

    # 定义项目标题
    API_TITLE = "Flyer Demo"

    # 定义接口path
    PREFIX = getenv("flyer_base_url", "/flyer")

    # 其他变量请参考BaseConfig内容



"""内置路由
"""
from fastapi import APIRouter
from fastapi.responses import JSONResponse, RedirectResponse
from fastapi.openapi.docs import get_redoc_html, get_swagger_ui_html
from fastflyer import status
from fastflyer import config

router = APIRouter(tags=["健康检查"])


@router.get("/", include_in_schema=False)
def root():  # NOQA
    """根路径访问自动跳转
    """
    return RedirectResponse(f"{config.PREFIX}/docs", status_code=302)


@router.get(f"{config.PREFIX}/health_check", include_in_schema=True)
@router.get("/health_check", include_in_schema=False)
def health_check():  # NOQA
    """健康检查接口
    """
    return JSONResponse(content={
        "message": "success",
        "code": status.HTTP_200_OK
    })


@router.get("/health_check/async", include_in_schema=False)
async def health_check_async():  # NOQA
    """健康检查接口-异步模式
    """
    return JSONResponse(content={
        "message": "success",
        "code": status.HTTP_200_OK
    })


@router.get(f"{config.PREFIX}/docs", include_in_schema=False)
def docs():  # NOQA
    """ Swagger UI
    """
    return get_swagger_ui_html(
        openapi_url=config.PREFIX + "/openapi.json",
        title=config.API_TITLE + "接口交互文档",
        swagger_js_url=config.PREFIX +
        "/static/swagger-ui-bundle.js?ver=5.1.0",
        swagger_css_url=config.PREFIX + "/static/swagger-ui.css?ver=5.1.0",
        swagger_favicon_url=config.PREFIX + "/static/favicon.ico")


@router.get(f"{config.PREFIX}/redoc", include_in_schema=False)
def redoc():
    """ ReDoc UI
    """
    return get_redoc_html(
        openapi_url=config.PREFIX + "/openapi.json",
        title=config.API_TITLE + "接口交互文档",
        redoc_js_url=config.PREFIX + "/static/redoc.standalone.js",
        redoc_favicon_url=config.PREFIX + "/static/favicon.ico",
        with_google_fonts=False)



# -*- coding: utf-8 -*-
from os import getenv


class BaseConfig:
    # 基本配置
    VERSION = getenv("flyer_version", "v1")
    PREFIX = getenv("flyer_base_url", "/flyer")
    DEVELOPER = getenv("flyer_author", "<请通过定义环境变量 flyer_author 来指定>")
    # 支持显示发布日期
    RELEASE_DATE = getenv("flyer_release_date",
                          "<请通过构建生成环境变量 flyer_release_date 来指定>")

    API_TITLE = "FastFlyer Demo"
    DESCRIPTION = f"中文名称：FastFlyer API 框架演示项目<br>\
功能说明：用于演示 FastFlyer API 开发框架的示例项目<br>\
框架源码：[Git](https://github.com/jagerzhang/fastflyer)<br>\
接口文档：[ReDoc]({PREFIX}/redoc)<br>\
快速上手：[SwaggerUI]({PREFIX}/docs)<br>\
技术支持：{DEVELOPER}<br>\
最新发布：{RELEASE_DATE}"

    # 本地日志是否开启颜色
    LOG_COLOR = True
    # 本地日志目录
    LOG_PATH = getenv("flyer_log_path", "/var/log")
    # 本地日志格式
    LOG_FORMAT = "%(asctime)s.%(msecs)03d | %(levelname)s | %(module)s:%(lineno)d | %(message)s"
    # 智研日志拓展字段，名称和logging的格式字段保持一致，支持已冒号的方式自定义别名，比如 dirname:business，则日志汇将展示 为 business
    ZHIYAN_LOG_EXT_FIELDS = ["location"]

    # 定义Body返回码的基数，默认值为0时，返回码和HTTP返回码保持一致
    STATUS_CODE_BASE = 0
    # 默认HTTP重试配置
    DEFAULT_REQUEST_RETRY_CONFIG = {
        "stop_max_attempt_number": 3,  # 最大重试 3 次
        "stop_max_delay": 60,  # 最大重试耗时 60 s
        "wait_exponential_multiplier": 2,  # 重试间隔时间倍数 2s、4s、8s...
        "wait_exponential_max": 10  # 最大重试间隔时间 10s
    }



# -*- coding: utf-8 -*-
"""
参数验证模块
"""
from typing import Optional, Union
from pydantic import BaseModel, Field
from fastflyer import status


class BaseResponse(BaseModel):
    """ Demo演示：响应参数.
    """
    code: int = Field(
        default=status.HTTP_200_OK,
        example=status.HTTP_200_OK,  # NOQA
        title="返回码",
        description="返回码，0表示成功，其他表示失败")
    message: str = Field(default="", example="", title="异常或提示信息")
    detail: str = Field(default="", example="", title="详细的异常或提示信息")


class PageInfo(BaseModel):
    """
    分页数据定义

    Args:
        BaseResponse (_type_): _description_
    """
    total: int = Field(
        default=0,
        example=0,  # NOQA
        title="返回数据总条数")
    offset: int = Field(
        default=1,
        example=1,  # NOQA
        title="返回数据的分页位置，用于按批次连续拉取")
    size: int = Field(
        default=50,
        example=50,  # NOQA
        title="返回数据的分页大小，默认为50")


class PageDataResponse(BaseResponse):
    """
    带分页数据响应

    Args:
        BaseResponse (_type_): _description_
    """
    pageInfo: Optional[PageInfo] = Field(default={}, title="返回数据的页面信息")
    data: Union[list, dict] = Field(default=[], example=[], title="接口返回的详细数据")


class DataResponse(BaseResponse):
    """
    数据通用响应

    Args:
        BaseResponse (_type_): _description_
    """
    data: Union[list, dict] = Field(default=[], example=[], title="接口返回的详细数据")



# -*- coding: utf-8 -*-
"""
参数定义
"""
from typing import List
from pydantic import BaseModel, Field


class UserInfo(BaseModel):
    """ User信息字段定义
    """
    name: str = Field(example="张三", title="姓名")
    age: str = Field(example="35", title="年龄")
    sex: str = Field(example="男", title="性别")
    father: str = Field(example="", title="父亲")
    mother: str = Field(example="", title="母亲")


class UserCreateRequest(BaseModel):
    """ 创建User：请求参数.
    """
    data: List[UserInfo] = Field(title="返回用户信息列表")



import secrets
from fastapi import Depends, status, HTTPException
from fastapi.security import HTTPBasic, HTTPBasicCredentials
from fastflyer.settings import BaseConfig as config

security = HTTPBasic()


def authorize(credentials: HTTPBasicCredentials = Depends(security)):
    """支持SwaggerUI开启BasicAuth鉴权
    """
    is_user_ok = secrets.compare_digest(credentials.username,
                                        config.env.get("flyer_auth_user"))
    is_pass_ok = secrets.compare_digest(credentials.password,
                                        config.env.get("flyer_auth_pass"))
    if not (is_user_ok and is_pass_ok):
        raise HTTPException(status_code=status.HTTP_401_UNAUTHORIZED,
                            detail="Incorrect user or password.",
                            headers={"WWW-Authenticate": "Basic"})



import os
from fastkit.discovery import Polaris
from fastflyer.utils import get_host_ip, get_log_path, get_env_list

host_ip = get_host_ip()
log_path = get_log_path()
flyer_polaris_enabled = int(os.getenv("flyer_polaris_enabled", 0))
polaris_namespace = os.getenv("flyer_polaris_namespace",
                              os.getenv("POLARIS_NAMESPACE"))
polaris_service_name = os.getenv("flyer_polaris_service",
                                 os.getenv("POLARIS_NAME"))
polaris_service_token = os.getenv("flyer_polaris_token",
                                  os.getenv("POLARIS_TOKEN"))
polaris_heartbeat_interval = int(
    os.getenv("flyer_polaris_heartbeat_interval", 5))
polaris_service_port = int(os.getenv("flyer_port", 8080))
polaris_metadata = get_env_list(prefix="flyer_polaris_metadata_", replace=True)

polaris = Polaris(namespace=polaris_namespace,
                  service=polaris_service_name,
                  service_token=polaris_service_token,
                  host=host_ip,
                  port=polaris_service_port,
                  metadata=polaris_metadata,
                  heartbeat_interval=polaris_heartbeat_interval,
                  misfire_grace_time=10,
                  max_instances=1000,
                  log_path=log_path) if flyer_polaris_enabled == 1 else None



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import os
from setuptools import find_packages, setup

settings = dict()
root_path = os.path.dirname(os.path.realpath(__file__))

with open(f"{root_path}/fastflyer/__info__.py", "r",
          encoding="utf-8") as version:
    setup_info = version.read()
    exec(setup_info)

with open(f"{root_path}/README.md", "r", encoding="utf-8") as file_readme:
    readme = file_readme.read()

with open(f"{root_path}/requirements.txt", "r") as file_requirements:
    requirements = file_requirements.read().splitlines()

settings.update(name=__package_name__,
                version=__version__,
                description=__title__,
                include_package_data=True,
                long_description_content_type="text/markdown",
                long_description=readme,
                author=__author__,
                author_email=__author_email__,
                license=__license__,
                url=__url__,
                packages=find_packages(),
                python_requires=__python_requires__,
                install_requires=requirements,
                zip_safe=True,
                entry_points={
                    "console_scripts":
                    ["fastflyer = fastflyer.tools.entrypoint:main"],
                },
                classifiers=__classifiers__)

setup(**settings)



# -*- coding: utf-8 -*-
"""
参数定义
"""
from pydantic import BaseModel, Field


class DemoRequest(BaseModel):
    """ Demo演示：请求参数.
    """
    msgContent: str = Field(example="Flyer", title="Flyer演示项目")



# -*- coding: utf-8 -*-
import re
import time
import json
from uuid import uuid4
import resource
from os import getenv
from fastapi.routing import APIRoute
from fastapi import Request, Response
from opentelemetry import trace
from fastflyer.utils import get_host_ip, get_client_ip
from fastflyer import logger

host_ip = get_host_ip()
# 最大body日志长度
max_body_log_length = int(getenv("flyer_max_body_log_length", "4096"))
# 最大参数集日志长度
max_params_log_length = int(getenv("flyer_max_params_log_length", "2048"))
# 最大header集日志长度
max_headers_log_length = int(getenv("flyer_max_headers_log_length", "2048"))
# 自动识别为json而无需传递content-type（如果默认，则客户端无需传递这个头部，否则必须传入才认为是JSON请求）
accept_json_without_content_type = int(getenv("flyer_accept_json_without_content_type", "0"))


class MiddleWare(APIRoute):
    """路由中间件：记录请求日志和耗时等公共处理逻辑"""

    async def _get_request_body(self, request: Request):
        """获取请求参数"""
        if request.headers.get("content-type", "").lower() == "application/json":
            try:
                return await request.json()
            except json.decoder.JSONDecodeError:
                pass

        return await request.body()

    async def _report_log(
        self,
        request_body: dict,
        request: Request,
        response: Response,
        client_ip: str,
        client_id: str,
        request_id: str,
        latency: int,
        memory_usage: int,
    ):
        """打印日志"""
        if int(getenv("flyer_access_log", "1")) == 0:
            return

        request_body = request_body or await self._get_request_body(request)
        response_body = getattr(response, "body", None) or ""
        access_log = {
            "direction": "in",
            "request": {
                "method": str(request.method).upper(),
                "url": str(request.url),
                "body": self._dict_to_str(request_body, max_body_log_length),
                "headers": self._dict_to_str(dict(request.headers), max_headers_log_length),
                "params": self._dict_to_str(request.query_params._dict, max_params_log_length),
            },
            "response": {
                "status": response.status_code,
                "body": self._dict_to_str(response_body, max_body_log_length),
                "headers": self._dict_to_str(dict(response.headers), max_headers_log_length),
            },
            "latency": latency,
            "clientIp": client_ip,
            "clientId": str(client_id),
            "memoryUsage": memory_usage,
            "logId": request_id,
        }

        logger_report = logger.warning if response.status_code >= 400 else logger.info
        try:
            logger_report(json.dumps(access_log, ensure_ascii=False))
        except Exception:  # pylint: disable=broad-except
            logger_report(str(access_log))

    def _dict_to_str(self, data: any, max_length: int = None) -> str:
        """将字典转成字符串"""
        try:
            if isinstance(data, (dict, list, tuple)):
                result = json.dumps(data, ensure_ascii=False)
            elif isinstance(data, bytes):
                result = bytes.decode(data)
            else:
                result = str(data)
        except Exception:
            result = str(data)

        return result[:max_length] if max_length is not None else result

    def get_route_handler(self):
        original_route_handler = super().get_route_handler()

        async def recorder(request: Request) -> Response:
            start_time = time.perf_counter()
            # 计算线程内存占用
            memory_usage_begin = resource.getrusage(resource.RUSAGE_THREAD).ru_maxrss

            # 对接NGate网关记录客户端ID
            client_id = request.headers.get("x-client-id", "")
            request_id = request.headers.get("x-request-id") or str(uuid4())
            client_ip = get_client_ip(request)
            # FastAPI 0.68.2 开始必须要传入 application/json 才识别为字典，这里制作一个开关，方便业务选择
            # https://github.com/tiangolo/fastapi/releases/tag/0.65.2
            request_body = None
            if accept_json_without_content_type == 1:
                # 注入之前就获取 body，减少except
                request_body = await self._get_request_body(request)
                request.headers.__dict__["_list"].insert(0, (b"content-type", b"application/json"))

            # 传递 x-request-id 到上游服务
            request.headers.__dict__["_list"].insert(0, (b"x-request-id", request_id.encode()))

            # opentelemetry 植入 x-request-id
            try:
                current_span = trace.get_current_span()
                if current_span:
                    current_span.set_attribute("http.request_id", request_id)
            except Exception:  # pylint: disable=broad-except
                pass

            response: Response = await original_route_handler(request)

            # 插入自定义头部
            memory_usage_end = resource.getrusage(resource.RUSAGE_THREAD).ru_maxrss
            memory_usage = memory_usage_end - memory_usage_begin
            latency = int((time.perf_counter() - start_time) * 1000)
            response.headers["X-Lasting-Time"] = str(latency)
            response.headers["X-Request-ID"] = request_id
            response.headers["X-Host-IP"] = host_ip
            response.headers["X-Memory-Usage"] = f"{memory_usage}KB"
            # 防御 XSS 反射型漏洞
            response.headers["X-Content-Type-Options"] = "nosniff"

            # 上报日志
            await self._report_log(
                request_body,
                request,
                response,
                client_ip,
                client_id,
                request_id,
                latency,
                memory_usage,
            )

            return response

        return recorder



import os
import sys
import importlib
import traceback
import asyncio
from pathlib import Path
from concurrent.futures import ThreadPoolExecutor
from fastapi import FastAPI, Depends
from fastapi.staticfiles import StaticFiles
from contextlib import asynccontextmanager
from opentelemetry.instrumentation.fastapi import FastAPIInstrumentor
from fastflyer.exceptions import init_exception
from fastflyer import config, background_scheduler, asyncio_scheduler, threadpool, tracer
from fastflyer.trace import fastapi_client_request_hook, fastapi_client_response_hook, fastapi_server_request_hook
from fastflyer.base.docs import router as docs_router
from fastflyer.base.tasks import router as tasks_router
from fastflyer.authorize import authorize
from fastkit.logging import get_logger

static_dir = os.path.join(os.path.dirname(__file__), "static")
logger = get_logger(logger_name="console", log_path=config.LOG_PATH)


class FlyerAPI:
    _inst = None
    middlewares = []

    def __new__(cls, app_path, middlewares: list = None):
        """
        规避重复加载
        """
        if cls._inst is None:
            cls.app_path = app_path
            if middlewares is not None:
                cls.middlewares = middlewares
            cls._inst = cls.create_app()

        return cls._inst

    @classmethod
    def create_app(cls):
        """创建应用"""
        cls.app = FastAPI(
            title=config.API_TITLE,
            description=config.DESCRIPTION,
            version=config.VERSION,
            openapi_url=config.PREFIX + "/openapi.json",
            docs_url=None,
            redoc_url=None,
            lifespan=lifespan,
        )
        cls.app.mount(config.PREFIX + "/static", StaticFiles(directory=static_dir), name="static")

        # 加载opentelemetry
        if tracer:
            FastAPIInstrumentor.instrument_app(
                cls.app,
                tracer_provider=tracer,
                server_request_hook=fastapi_server_request_hook,
                client_request_hook=fastapi_client_request_hook,
                client_response_hook=fastapi_client_response_hook,
            )

        # 加载文档路由
        cls.app.include_router(docs_router)

        # 加载自定义中间件
        for middleware in cls.middlewares:
            cls.app.middleware("http")(middleware)
        # 自动加载子项目
        cls.load_module()
        # 加载任务管理路由
        cls.app.include_router(tasks_router)
        # 初始化异常处理
        init_exception(cls.app)
        return cls.app

    @classmethod
    def check_auth_path(path: str) -> bool:
        """检查是否需要鉴权"""
        if not path:
            return True
        white_list = os.getenv("flyer_no_auth_path_prefix", "").split(",")
        for prefix in white_list:
            for r in router.routes:
                path = getattr(r, "path")
                if path.startswith(prefix):
                    return False
        return True

    @classmethod
    def load_module(cls):
        """
        加载子项目

        Args:
            app (FastAPI): FastAPI对象
        """
        # 获取指定项目的根目录，即app目录的上一层
        root_path = os.path.dirname(cls.app_path)
        # 获取app目录名称
        app_dir = os.path.basename(cls.app_path)
        # 进入项目根目录
        sys.path.append(root_path)
        os.chdir(root_path)
        # 自动加载有路由的包
        for _dir in Path(app_dir).iterdir():
            # 不存在则跳过
            if not _dir.exists():
                continue

            # 隐藏文件夹或申明非开放文件夹或不是文件夹的跳过
            if _dir.name.startswith("_") or _dir.name.startswith(".") or not _dir.is_dir():
                continue

            try:
                sub_module = importlib.import_module(f"{app_dir}.{_dir.name}")
                # 尝试获取子模块是否启用，默认为True
                sub_module_enabled = getattr(sub_module, "__ENABLED__", True)
                if not sub_module_enabled:
                    continue

                # 跳过无路由的模块
                if not hasattr(sub_module, "router"):
                    logger.info(f"后台子项目加载成功：{_dir.name}")
                    continue

                # 开启 BasicAuth 鉴权
                if int(os.getenv("flyer_auth_enable", "0")) == 1 and cls.check_auth_path(sub_module.router.routes):
                    cls.app.include_router(
                        sub_module.router, prefix=f"{config.PREFIX}", dependencies=[Depends(authorize)]
                    )

                else:
                    cls.app.include_router(sub_module.router, prefix=f"{config.PREFIX}")

                logger.info(f"API子项目加载成功：{_dir.name}")

            except Exception:  # pylint: disable=broad-except
                logger.error(f"API子路由加载错误：{traceback.format_exc()}")
                pass


@asynccontextmanager
async def lifespan(app: FastAPI):
    """生命周期管理"""

    def on_startup():
        """启动时执行逻辑"""
        max_threads = int(os.environ.get("flyer_threads", 5))
        logger.info(f"The Number of Threads per Worker: {max_threads}")
        loop = asyncio._get_running_loop()
        loop.set_default_executor(ThreadPoolExecutor(max_workers=max_threads))

        # 启动任务调度
        if not background_scheduler.running:
            logger.info("正在启动 Apscheduler 后台线程任务...")
            background_scheduler.start()

        # 启动异步任务调度
        if not asyncio_scheduler.running:
            logger.info("正在启动 Apscheduler 后台协程任务...")
            asyncio_scheduler.start()

    def on_shutdown():
        """关闭时执行逻辑"""
        logger.warn("收到关闭信号，FastFlyer 开始执行优雅停止逻辑...")
        # 等待一段时间再退出（最小1秒，最大60秒）
        graceful_timeout = max(min(int(os.getenv("flyer_graceful_timeout", "1")), 60), 1)
        # 关闭任务调度器
        logger.info("正在关闭 Apscheduler 定时引擎...")
        background_scheduler.shutdown(wait=True, timeout=graceful_timeout)
        asyncio_scheduler.shutdown(wait=True, timeout=graceful_timeout)
        # 关闭线程池
        logger.warning(f"正在关闭线程池，至多等待 {graceful_timeout}s ...")
        threadpool.shutdown(wait=True, timeout=graceful_timeout)
        logger.info("FastFlyer 已成功停止服务，感谢您的使用，再见！")

    on_startup()
    yield
    on_shutdown()
